import java.awt.event.*;
import java.awt.event.MouseAdapter;
import java.awt.*;
/**
 * Clase que redefine las acciones realizadas por el mouse.
 */
public class MouseListener extends MouseAdapter {
   private BlockSpringConfiguration bsConfig;
   private Draggable currentElement;
   private LabPanel labPanel;
   private SelectListener sl;
   /**
    * Construye el MouseListener con los objetos utilizados para la simlacion.
    * @param bsc Configuracion de elementos fisicos.
    * @param lp Panel al que estara sujeto el listener.
    */
   public MouseListener (BlockSpringConfiguration bsc, LabPanel lp, SelectListener sListener){
      bsConfig = bsc;
      labPanel = lp;
      sl = sListener;
   }
   /**
    * Accion a seguir cuando se mueve el mouse.
    * @param e Evento del mouse.
    */
   public void mouseMoved(MouseEvent e) {
      Draggable newElement = bsConfig.find(e.getX(), e.getY());
      if (newElement == currentElement) return;
      if (currentElement != null) {
         currentElement.setDefaultColor();
         currentElement = null;
         labPanel.repaint();
      }
      if (newElement != null) { 
         currentElement = newElement;
         currentElement.setSelectedColor();
         labPanel.repaint();
      }
   }
    /**
    * Accion a seguir cuando se arrastra un elemento fisico.
    * @param e Evento del mouse.
    */
   public void mouseDragged(MouseEvent e) {
      if (currentElement == null) return;
      currentElement.drag(e.getX(), e.getY());
      labPanel.repaint();
   }
    /**
    * Accion a seguir cuando se libera el boton del mouse.
    * @param e Evento del mouse.
    */
   public void mouseReleased(MouseEvent e) {
      if (currentElement == null) return;
      currentElement.attachNearElement(e.getX(), e.getY(), bsConfig);
      currentElement.setDefaultColor();          
      if (currentElement instanceof Block)
          sl.blockClicked((Block)currentElement);
      labPanel.repaint();
      currentElement = null;
   }
	
}