import java.util.*;
import javax.swing.*;
import java.awt.Container;
import java.io.*;
/**
 * Clase principal de la aplicacion del laboratorio de elementos fisicos.
 */
public class PhysicsLab {
    /**
     * Metodo que implementa la clase principal de la aplicacion del laboratorio de elementos fisicos.
     * @param args argumentos de inicio, no se toman en consideracion en la ejecucion.
     */
   public static void main(String[] args) {   
      LabFrame labFrame = new LabFrame();
     labFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      labFrame.setVisible(true);
   }
}
/**
 * Clase que inicia el frame grafico de la aplicacion e inicia la simulacion.
 */
class LabFrame extends JFrame {
   public LabFrame() {

      setTitle("Blocks, Springs and Strings Laboratory");
      setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);
      GraphPanel graphPanel = new GraphPanel();
      BlockSpringConfiguration bsConfig = new BlockSpringConfiguration();
      LabPanel labPanel = new LabPanel(bsConfig);
      graphPanel.setAlignmentY(LEFT_ALIGNMENT);
      JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, labPanel, graphPanel);
      splitPane.setDividerLocation((int)(0.6*DEFAULT_WIDTH));
      bsConfig.setLabPanel(labPanel);   // to repaint view when model changes
      LabMenuBar menuBar = null;
      InsertElementListener ieListener = new InsertElementListener(bsConfig);
      Simulator sim = new Simulator(bsConfig,graphPanel);
      SelectListener selListener = new SelectListener(sim);
      SimulatorListener sListener = new SimulatorListener(sim, graphPanel);
      menuBar = new LabMenuBar(ieListener, sListener,selListener);
      ieListener.setMenu(menuBar);
      MouseListener mListener = new MouseListener(bsConfig, labPanel, selListener);
      labPanel.addMouseMotionListener(mListener);
      labPanel.addMouseListener(mListener);
      Container contentPane = getContentPane();
      contentPane.add(splitPane);
      setJMenuBar(menuBar);

      //Lectura de propidades y seteo de ellas

      try {
            NumberFormatException b = new NumberFormatException("Negative number");
            FileInputStream file = new FileInputStream("PhysicsLab.ini");
            Properties prop = new Properties();
            prop.load(file);

            try{
                if(Float.parseFloat(prop.getProperty("DeltaTime")) > 0){
                  sim.setDelta_t(Float.parseFloat(prop.getProperty("DeltaTime")));
                } else throw(b);
            } catch (NumberFormatException a){
                     JOptionPane.showMessageDialog(this, "Invalid parameter found for delta time, using default value", "Error", JOptionPane.ERROR_MESSAGE);
            } catch (NullPointerException a){
            }

            try{
                  if(Float.parseFloat(prop.getProperty("RefreshTime")) > 0){
                      sim.setSamplingPeriod(Float.parseFloat(prop.getProperty("RefreshTime")));
                  } else throw(b);
            } catch (NumberFormatException a){
                     JOptionPane.showMessageDialog(this, "Invalid parameter found for refresh time, using default value", "Error", JOptionPane.ERROR_MESSAGE);
            } catch (NullPointerException a){
            }

            try{
                  if(Double.parseDouble(prop.getProperty("timeWindowSize")) > 0){
                      graphPanel.setMaxTime(Double.parseDouble(prop.getProperty("timeWindowSize")));
                  } else throw(b);
            } catch (NumberFormatException a){
                    JOptionPane.showMessageDialog(this, "Invalid parameter found for max time, using default value", "Error", JOptionPane.ERROR_MESSAGE);
            } catch (NullPointerException a){
            }

            try{
                  if(Double.parseDouble(prop.getProperty("Gravity")) > 0){
                      sim.setGravity(Float.parseFloat(prop.getProperty("Gravity")));
                  } else throw(b);
            } catch (NumberFormatException a){
                    JOptionPane.showMessageDialog(this, "Invalid parameter found for gravity, using default value", "Error", JOptionPane.ERROR_MESSAGE);
            } catch (NullPointerException a){
            }

      } catch (FileNotFoundException ex) {
            JOptionPane.showMessageDialog(this, "PhysicsLab.ini not found, using default values", "Error", JOptionPane.ERROR_MESSAGE);
      } catch (IOException ex) {
          JOptionPane.showMessageDialog(this, "Couldnt read PhysicsLab.ini, using default values", "Error", JOptionPane.ERROR_MESSAGE);
      }
   }

   public static final int DEFAULT_WIDTH = 800;
   public static final int DEFAULT_HEIGHT = 600;
}
